#ifndef SYSTEM_CALLS_LIST_H
#define SYSTEM_CALLS_LIST_H

#include <string>

/**
 * The count of system calls.
 * @see systemCallMappings
 */
const int SYSTEM_CALL_COUNT = 463;

/**
 * A list of system calls. This information was scraped from
 * <sys/syscall.h> using the syscalls Rust crate:
 *
 * $ git clone https://github.com/wangbj/syscalls && cd syscalls/
 * $ git checkout ef90cdfb1f31a9601ed44a40e737b189edf1c7c4
 * $ rustup install nightly-2019-10-24
 * $ rustup run nightly-2019-10-24 cargo build
 * $ find . -iname "nr.rs" | xargs cat
 *
 * The most recent information in this list was determined by running
 * the above script on an Ubuntu 18.04 LTS (Linux version 4.15.0-54-generic)
 * machine. If you are using a more recent version of the Linux kernel,
 * you may have additional system calls that are not present in this list.
 * If so, please run the script and add them!
 */
const std::string systemCallMappings[SYSTEM_CALL_COUNT] = {
    "read",
    "write",
    "open",
    "close",
    "stat",
    "fstat",
    "lstat",
    "poll",
    "lseek",
    "mmap",
    "mprotect",
    "munmap",
    "brk",
    "rt_sigaction",
    "rt_sigprocmask",
    "rt_sigreturn",
    "ioctl",
    "pread64",
    "pwrite64",
    "readv",
    "writev",
    "access",
    "pipe",
    "select",
    "sched_yield",
    "mremap",
    "msync",
    "mincore",
    "madvise",
    "shmget",
    "shmat",
    "shmctl",
    "dup",
    "dup2",
    "pause",
    "nanosleep",
    "getitimer",
    "alarm",
    "setitimer",
    "getpid",
    "sendfile",
    "socket",
    "connect",
    "accept",
    "sendto",
    "recvfrom",
    "sendmsg",
    "recvmsg",
    "shutdown",
    "bind",
    "listen",
    "getsockname",
    "getpeername",
    "socketpair",
    "setsockopt",
    "getsockopt",
    "clone",
    "fork",
    "vfork",
    "execve",
    "exit",
    "wait4",
    "kill",
    "uname",
    "semget",
    "semop",
    "semctl",
    "shmdt",
    "msgget",
    "msgsnd",
    "msgrcv",
    "msgctl",
    "fcntl",
    "flock",
    "fsync",
    "fdatasync",
    "truncate",
    "ftruncate",
    "getdents",
    "getcwd",
    "chdir",
    "fchdir",
    "rename",
    "mkdir",
    "rmdir",
    "creat",
    "link",
    "unlink",
    "symlink",
    "readlink",
    "chmod",
    "fchmod",
    "chown",
    "fchown",
    "lchown",
    "umask",
    "gettimeofday",
    "getrlimit",
    "getrusage",
    "sysinfo",
    "times",
    "ptrace",
    "getuid",
    "syslog",
    "getgid",
    "setuid",
    "setgid",
    "geteuid",
    "getegid",
    "setpgid",
    "getppid",
    "getpgrp",
    "setsid",
    "setreuid",
    "setregid",
    "getgroups",
    "setgroups",
    "setresuid",
    "getresuid",
    "setresgid",
    "getresgid",
    "getpgid",
    "setfsuid",
    "setfsgid",
    "getsid",
    "capget",
    "capset",
    "rt_sigpending",
    "rt_sigtimedwait",
    "rt_sigqueueinfo",
    "rt_sigsuspend",
    "sigaltstack",
    "utime",
    "mknod",
    "uselib",
    "personality",
    "ustat",
    "statfs",
    "fstatfs",
    "sysfs",
    "getpriority",
    "setpriority",
    "sched_setparam",
    "sched_getparam",
    "sched_setscheduler",
    "sched_getscheduler",
    "sched_get_priority_max",
    "sched_get_priority_min",
    "sched_rr_get_interval",
    "mlock",
    "munlock",
    "mlockall",
    "munlockall",
    "vhangup",
    "modify_ldt",
    "pivot_root",
    "_sysctl",
    "prctl",
    "arch_prctl",
    "adjtimex",
    "setrlimit",
    "chroot",
    "sync",
    "acct",
    "settimeofday",
    "mount",
    "umount2",
    "swapon",
    "swapoff",
    "reboot",
    "sethostname",
    "setdomainname",
    "iopl",
    "ioperm",
    "create_module",
    "init_module",
    "delete_module",
    "get_kernel_syms",
    "query_module",
    "quotactl",
    "nfsservctl",
    "getpmsg",
    "putpmsg",
    "afs_syscall",
    "tuxcall",
    "security",
    "gettid",
    "readahead",
    "setxattr",
    "lsetxattr",
    "fsetxattr",
    "getxattr",
    "lgetxattr",
    "fgetxattr",
    "listxattr",
    "llistxattr",
    "flistxattr",
    "removexattr",
    "lremovexattr",
    "fremovexattr",
    "tkill",
    "time",
    "futex",
    "sched_setaffinity",
    "sched_getaffinity",
    "set_thread_area",
    "io_setup",
    "io_destroy",
    "io_getevents",
    "io_submit",
    "io_cancel",
    "get_thread_area",
    "lookup_dcookie",
    "epoll_create",
    "epoll_ctl_old",
    "epoll_wait_old",
    "remap_file_pages",
    "getdents64",
    "set_tid_address",
    "restart_syscall",
    "semtimedop",
    "fadvise64",
    "timer_create",
    "timer_settime",
    "timer_gettime",
    "timer_getoverrun",
    "timer_delete",
    "clock_settime",
    "clock_gettime",
    "clock_getres",
    "clock_nanosleep",
    "exit_group",
    "epoll_wait",
    "epoll_ctl",
    "tgkill",
    "utimes",
    "vserver",
    "mbind",
    "set_mempolicy",
    "get_mempolicy",
    "mq_open",
    "mq_unlink",
    "mq_timedsend",
    "mq_timedreceive",
    "mq_notify",
    "mq_getsetattr",
    "kexec_load",
    "waitid",
    "add_key",
    "request_key",
    "keyctl",
    "ioprio_set",
    "ioprio_get",
    "inotify_init",
    "inotify_add_watch",
    "inotify_rm_watch",
    "migrate_pages",
    "openat",
    "mkdirat",
    "mknodat",
    "fchownat",
    "futimesat",
    "newfstatat",
    "unlinkat",
    "renameat",
    "linkat",
    "symlinkat",
    "readlinkat",
    "fchmodat",
    "faccessat",
    "pselect6",
    "ppoll",
    "unshare",
    "set_robust_list",
    "get_robust_list",
    "splice",
    "tee",
    "sync_file_range",
    "vmsplice",
    "move_pages",
    "utimensat",
    "epoll_pwait",
    "signalfd",
    "timerfd_create",
    "eventfd",
    "fallocate",
    "timerfd_settime",
    "timerfd_gettime",
    "accept4",
    "signalfd4",
    "eventfd2",
    "epoll_create1",
    "dup3",
    "pipe2",
    "inotify_init1",
    "preadv",
    "pwritev",
    "rt_tgsigqueueinfo",
    "perf_event_open",
    "recvmmsg",
    "fanotify_init",
    "fanotify_mark",
    "prlimit64",
    "name_to_handle_at",
    "open_by_handle_at",
    "clock_adjtime",
    "syncfs",
    "sendmmsg",
    "setns",
    "getcpu",
    "process_vm_readv",
    "process_vm_writev",
    "kcmp",
    "finit_module",
    "sched_setattr",
    "sched_getattr",
    "renameat2",
    "seccomp",
    "getrandom",
    "memfd_create",
    "kexec_file_load",
    "bpf",
    "execveat",
    "userfaultfd",
    "membarrier",
    "mlock2",
    "copy_file_range",
    "preadv2",
    "pwritev2",
    "pkey_mprotect",
    "pkey_alloc",
    "pkey_free",
    "statx",
    "io_pgetevents",
    "rseq",
    "uretprobe",
    "dummy336",
    "dummy337",
    "dummy338",
    "dummy339",
    "dummy340",
    "dummy341",
    "dummy342",
    "dummy343",
    "dummy344",
    "dummy345",
    "dummy346",
    "dummy347",
    "dummy348",
    "dummy349",
    "dummy350",
    "dummy351",
    "dummy352",
    "dummy353",
    "dummy354",
    "dummy355",
    "dummy356",
    "dummy357",
    "dummy358",
    "dummy359",
    "dummy360",
    "dummy361",
    "dummy362",
    "dummy363",
    "dummy364",
    "dummy365",
    "dummy366",
    "dummy367",
    "dummy368",
    "dummy369",
    "dummy370",
    "dummy371",
    "dummy372",
    "dummy373",
    "dummy374",
    "dummy375",
    "dummy376",
    "dummy377",
    "dummy378",
    "dummy379",
    "dummy380",
    "dummy381",
    "dummy382",
    "dummy383",
    "dummy384",
    "dummy385",
    "dummy386",
    "dummy387",
    "dummy388",
    "dummy389",
    "dummy390",
    "dummy391",
    "dummy392",
    "dummy393",
    "dummy394",
    "dummy395",
    "dummy396",
    "dummy397",
    "dummy398",
    "dummy399",
    "dummy400",
    "dummy401",
    "dummy402",
    "dummy403",
    "dummy404",
    "dummy405",
    "dummy406",
    "dummy407",
    "dummy408",
    "dummy409",
    "dummy410",
    "dummy411",
    "dummy412",
    "dummy413",
    "dummy414",
    "dummy415",
    "dummy416",
    "dummy417",
    "dummy418",
    "dummy419",
    "dummy420",
    "dummy421",
    "dummy422",
    "dummy423",
    "pidfd_send_signal",
    "io_uring_setup",
    "io_uring_enter",
    "io_uring_register",
    "open_tree",
    "move_mount",
    "fsopen",
    "fsconfig",
    "fsmount",
    "fspick",
    "pidfd_open",
    "clone3",
    "close_range",
    "openat2",
    "pidfd_getfd",
    "faccessat2",
    "process_madvise",
    "epoll_pwait2",
    "mount_setattr",
    "quotactl_fd",
    "landlock_create_ruleset",
    "landlock_add_rule",
    "landlock_restrict_self",
    "memfd_secret",
    "process_mrelease",
    "futex_waitv",
    "set_mempolicy_home_node",
    "cachestat",
    "fchmodat2",
    "map_shadow_stack",
    "futex_wake",
    "futex_wait",
    "futex_requeue",
    "statmount",
    "listmount",
    "lsm_get_self_attr",
    "lsm_set_self_attr",
    "lsm_list_modules",
    "mseal"
    };

#endif
